#!perl

BEGIN {
  unless ($ENV{RELEASE_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for release candidate testing');
  }
}


# This test is generated by Dist::Zilla::Plugin::Test::Fixme
use strict;
use warnings;
use Test::More;

eval { require Test::Fixme; Test::Fixme->import(); };
plan(skip_all => "Test::Fixme not installed: $@; skipping") if $@;

run_tests();
